"""
One Piece API wrapper (french data)
"""
from .functions import *
from .interface import API

__all__ = [
    "API",
    "episode_by_id",
    "episode_by_title",
    "episode_by_saga_id",
    "episode_by_arc_id",
    "episode_by_arc_title",
    "episode_by_saga_title",
    "count_episodes",
    "movie_by_id",
    "movie_by_title",
    "count_movies",
    "tome_by_id",
    "tome_by_title",
    "count_tomes",
    "chapter_by_id",
    "chapter_by_title",
    "count_chapters",
    "arc_by_id",
    "arc_by_title",
    "arc_by_saga_id",
    "count_arcs",
    "saga_by_id",
    "saga_by_title",
    "count_sagas",
    "haki_by_id",
    "haki_by_name",
    "haki_by_roman_name",
    "haki_by_characters_id",
    "count_hakis",
    "character_by_id",
    "character_by_name",
    "character_by_job",
    "character_by_bounty",
    "character_by_status",
    "character_by_size",
    "count_characters",
    "dial_by_id",
    "dial_by_name",
    "dial_by_type",
    "count_dials",
    "luffy_gear_by_id",
    "luffy_gear_by_title",
    "count_luffy_gears",
    "luffy_technique_by_id",
    "luffy_technique_by_name",
    "luffy_technique_by_translation",
    "count_luffy_techniques",
    "locate_by_id",
    "locate_by_name",
    "locate_by_sea",
    "locate_by_affiliation",
    "count_locates",
    "fruit_by_id",
    "fruit_by_name",
    "fruit_by_type",
    "count_fruits",
    "sword_by_id",
    "sword_by_name",
    "count_swords",
    "boat_by_id",
    "boat_by_name",
    "boat_by_type",
    "boat_by_crew_id",
    "boat_by_captain_id",
    "count_boats",
    "crew_by_id",
    "crew_by_name",
    "crew_by_status",
    "count_crews",
]



import unittest

from onepyece import functions
from onepyece.common import ENDPOINTS, NO_RESOURCE_SEARCHES


class TestFunctions(unittest.TestCase):
    def test_functions(self):
        """Test if all functions are present for all endpoints and searches"""
        for endpoint in ENDPOINTS:
            for search in ENDPOINTS[endpoint]:
                current_endpoint = endpoint[:-1] if endpoint.endswith("s") else endpoint
                current_endpoint = current_endpoint.replace("/", "_")
                if search not in NO_RESOURCE_SEARCHES:
                    function_name = f"{current_endpoint}_by_{search}"
                else:
                    function_name = f"{search}_{current_endpoint}s"
                self.assertTrue(hasattr(functions, function_name))



import unittest

from onepyece import common, interface


class TestBuildURL(unittest.TestCase):
    def test_url_no_search(self, endpoint="characters"):
        url = common.build_url(endpoint)
        self.assertEqual(url, "https://api.api-onepiece.com/characters")

    def test_url_with_search(self, endpoint="characters", search="id", resource=1):
        url = common.build_url(endpoint, search, resource)
        self.assertEqual(url, "https://api.api-onepiece.com/characters/1")

    def test_url_with_search_and_count(self, endpoint="characters", search="count"):
        url = common.build_url(endpoint, search)
        self.assertEqual(url, "https://api.api-onepiece.com/characters/count")

    def test_url_with_search_and_resource_id(self, endpoint="characters", search="name", resource="Monkey D Luffy"):
        url = common.build_url(endpoint, search, resource)
        self.assertEqual(url, "https://api.api-onepiece.com/characters/search/name/Monkey%20D%20Luffy")

    def test_url_with_search_and_with_slash(self, endpoint="characters", search="name", resource="Baggy / Le Clown"):
        url = common.build_url(endpoint, search, resource)
        self.assertEqual(url, "https://api.api-onepiece.com/characters/search/name/Baggy")

    def test_url_search_with_slash_and_space(self, endpoint="characters", search="name", resource="Baggy / Le Clown "):
        url = common.build_url(endpoint, search, resource)
        self.assertEqual(url, "https://api.api-onepiece.com/characters/search/name/Baggy")

    def test_url_by_title(self, endpoint="episodes", search="title", resource="Je suis Luffy !"):
        url = common.build_url(endpoint, search, resource)
        self.assertEqual(url, "https://api.api-onepiece.com/episodes/search/Je%20suis%20Luffy%20!")

    def test_url_search_other_id(self, endpoint="boats", search="crew_id", resource=1):
        url = common.build_url(endpoint, search, resource)
        self.assertEqual(url, "https://api.api-onepiece.com/boats/search/crew/1")


class TestCheckParams(unittest.TestCase):
    """ Test the check_params function
    The None value is returned if the parameters are valid
    The ValueError is raised if the parameters are not valid
    """

    def test_endpoint(self, endpoint="characters"):
        self.assertEqual(common.check_params(endpoint), None)

    def test_endpoint_with_wrong_value(self, endpoint="wrong"):
        self.assertRaises(ValueError, common.check_params, endpoint)

    def test_search_no_resource(self, endpoint="characters", search="id"):
        self.assertRaises(ValueError, common.check_params, endpoint, search)

    def test_search_with_resource(self, endpoint="characters", search="id", resource=1):
        self.assertEqual(common.check_params(endpoint, search, resource), None)

    def test_search_with_wrong_value(self, endpoint="characters", search="wrong", resource=1):
        self.assertRaises(ValueError, common.check_params, endpoint, search, resource)


class TestConvertName(unittest.TestCase):
    def test_convert_resource(self, name="Baggy / Le Clown"):
        self.assertEqual(common.convert_resource(name), "Baggy")

    def test_convert_resource_with_space(self, name="Baggy / Le Clown "):
        self.assertEqual(common.convert_resource(name), "Baggy")


class TestEndpointsSearches(unittest.TestCase):
    def test_endpoints(self):
        """Testing if all combinations of endpoints and searches return some data.
        The ValueError means that resource doesnt exist but the endpoint and search are valid
        """
        resource = "123456798123456789"
        for endpoint in common.ENDPOINTS:
            for search in common.ENDPOINTS[endpoint]:
                self.assertRaises(ValueError, interface.API, endpoint, search, resource)



import unittest

from onepyece import interface


class TestAPI(unittest.TestCase):
    def test_unique_character(self):
        api_object = interface.API("characters", "id", 1)
        self.assertIsInstance(api_object, interface.API)
        self.assertEqual(api_object.resource, 1)
        self.assertEqual(api_object.id, 1)
        self.assertEqual(api_object.french_name, "Monkey D Luffy")

    def test_unique_character_with_wrong_resource_id(self):
        self.assertRaises(ValueError, interface.API, "characters", "id", "wrong")

    def test_unique_character_with_wrong_search(self):
        self.assertRaises(ValueError, interface.API, "characters", "wrong", 1)

    def test_unique_character_with_wrong_endpoint(self):
        self.assertRaises(ValueError, interface.API, "wrong", "id", 1)

    def test_multiple_result(self):
        api_object = interface.API("hakis", "name", "Haki")
        self.assertIsInstance(api_object, interface.API)
        self.assertIsInstance(api_object.results, list)
        self.assertEqual(api_object.resource, "Haki")
        self.assertRaises(AttributeError, getattr, api_object, "id")
        self.assertIsInstance(api_object.results[0], interface.API)
        self.assertEqual(api_object.results[0].french_name, "Haki de l'Observation")

    def test_count_implicit(self):
        api_object = interface.API("hakis")
        self.assertIsInstance(api_object, interface.API)
        self.assertEqual(api_object.count, 3)

    def test_count_implicit_with_search(self):
        api_object = interface.API("hakis", "name", "Haki")
        self.assertIsInstance(api_object, interface.API)
        self.assertEqual(api_object.count, 3)

    def test_count_explicit(self):
        api_object = interface.API("hakis", "count")
        self.assertIsInstance(api_object, interface.API)
        self.assertEqual(api_object.count, 3)

    def test_iterate_object(self):
        # Multiple results -> iterate
        api_object = interface.API("crews")
        self.assertIsInstance(api_object, interface.API)
        for result in api_object:
            self.assertIsInstance(result, interface.API)
        # Unique result -> do not iterate
        api_object = interface.API("crews", "id", 1)
        self.assertIsInstance(api_object, interface.API)
        self.assertRaises(TypeError, iter, api_object)

    def test_recursive_dict(self):
        api_object = interface.API('hakis', 'characters_id', 1)
        self.assertIsInstance(api_object[0].characters.haki_character, interface.API)

    def test_search_method(self):
        api_object = interface.API('characters').search('job', 'capitaine')
        self.assertEqual(api_object[1].french_name, 'Cavendish')

    def test_search_method_with_search(self):
        with self.assertRaises(ValueError):
            interface.API('characters', 'count').search('job', 'capitaine')

    def test_search_method_with_search_and_resource(self):
        with self.assertRaises(ValueError):
            interface.API('characters', 'status', 'décédé').search('job', 'capitaine')

    def test_edit_resource(self):
        api_object = interface.API('characters', 'name', 'luffy')
        self.assertEqual(api_object.french_name, 'Monkey D Luffy')
        api_object.edit_resource('zoro')
        self.assertEqual(api_object.french_name, 'Roronoa Zoro')

    def test_edit_resource_not_allowed(self):
        api_object = interface.API('characters', 'count')
        api_object_2 = interface.API('crews', 'yonko')
        with self.assertRaises(ValueError):
            api_object.edit_resource('zoro')
        with self.assertRaises(ValueError):
            api_object_2.edit_resource('zoro')

    def test_edit_resource_attributes(self):
        api_object = interface.API('characters', 'name', 'luffy')
        self.assertEqual(api_object.french_name, 'Monkey D Luffy')
        self.assertEqual(api_object.url, 'https://api.api-onepiece.com/characters/search/name/luffy')
        api_object.edit_resource('zoro')
        self.assertEqual(api_object.french_name, 'Roronoa Zoro')
        self.assertEqual(api_object.url, 'https://api.api-onepiece.com/characters/search/name/zoro')

    def test_edit_resource_with_wrong_resource(self):
        api_object = interface.API('characters', 'name', 'luffy')
        with self.assertRaises(ValueError):
            api_object.edit_resource('wrong')



import unittest

from onepyece import api


class TestGetData(unittest.TestCase):
    def test_get_data(self):
        data = api.get_data("https://api.api-onepiece.com/characters")
        self.assertIsInstance(data, list)

    def test_get_data_with_wrong_url(self):
        self.assertRaises(ValueError, api.get_data, "wrong")

    def test_get_data_empty_result(self):
        data = api.get_data("https://api.api-onepiece.com/characters/4444")
        self.assertIsNone(data)



"""
This module contains the functions to get data from the API
"""
import requests


def get_data(url):
    """
    Get data from the API

    See https://api-onepiece.com/documentation for more information

    :param url: The url to get the data from
    :return: The data from the API or None if no data is found
    """
    response = requests.get(url, timeout=5)
    response.raise_for_status()
    return None if response.json() is None or response.json() == [] else response.json()



"""
Common functions for the API
"""
import json

URL = "https://api.api-onepiece.com/"
ENDPOINTS = {
    "episodes": ["id", "count", "title", "saga_id", "arc_id", "arc_title", "saga_title"],
    "movies": ["id", "count", "title"],
    "tomes": ["id", "count", "title"],
    "chapters": ["id", "count", "title", "tome_id"],
    "arcs": ["id", "count", "title", "saga_id"],
    "sagas": ["id", "count", "title"],
    "hakis": ["id", "count", "name", "roman_name", "characters_id"],
    "characters": ["id", "count", "name", "job", "bounty", "status", "size", "crew_id", "fruit_id"],
    "dials": ["id", "count", "name", "type"],
    "luffy/gears": ["id", "count", "title"],
    "luffy/techniques": ["id", "count", "name", "translation", "gear_id"],
    "locates": ["id", "count", "name", "sea", "affiliation"],
    "fruits": ["id", "count", "type"],
    "swords": ["id", "count", "name", "destroy"],
    "boats": ["id", "count", "name", "type", "crew_id", "captain_id"],
    "crews": ["id", "count", "name", "status", "yonko"],
}

STRING_SEARCHES = ["name", "job", "bounty", "status", "size",
                   "type", "roman_name", "sea", "affiliation"]
ID_SEARCHES = ["crew_id", "captain_id"]
NO_SEARCH_ID_SEARCHES = ["saga_id", "arc_id", "characters_id",
                         "gear_id", "tome_id", "fruit_id", "crew_id"]
NO_RESOURCE_SEARCHES = ["count", "yonko"]


def check_params(endpoint, search=None, resource=None):
    """
    Check if the parameters are valid for the API

    :param endpoint: The endpoint to check
    :param search: The search to check
    :param resource: The resource to check

    See ENDPOINTS in common.py for more information

    This function is supposed to be used only by build_url

    :return: None if the parameters are valid
    :raises ValueError: If the parameters are not valid
    """
    if endpoint not in ENDPOINTS:
        raise ValueError(f"Unknown API endpoint '{endpoint}'")
    if search is not None and search not in ENDPOINTS[endpoint]:
        raise ValueError(f"Unknown search '{search}' for endpoint '{endpoint}'")
    if search is not None and search not in NO_RESOURCE_SEARCHES and resource is None:
        raise ValueError("Resource is required for this search")
    if search in [NO_RESOURCE_SEARCHES] and resource is not None:
        raise ValueError("Resource is not required for this search")
    if resource is not None and "id" in search and not isinstance(resource, int):
        raise ValueError("Resource must be an integer for this search")


def build_url(endpoint, search=None, resource=None):
    """
    Build the url for the API

    :param endpoint: The endpoint to build the url for
    :param search: The search to build the url for
    :param resource: The resource to build the url for

    See ENDPOINTS in common.py for more information

    :return: The url for the API
    """
    check_params(endpoint, search, resource)
    if search is not None:
        if resource is not None:
            resource = convert_resource(resource)
            return adding_search(endpoint, search, resource)
        return f"{URL}{endpoint}/{search}"
    return f"{URL}{endpoint}"


def adding_search(endpoint, search, resource=None):
    """
    Add the search to the url

    See https://api-onepiece.com/documentation for more information about URL

    This functions is supposed to be used only by build_url

    :param endpoint: The endpoint to add the search to
    :param search: The search to add
    :param resource: The resource to add

    :return: The url with the search added
    """
    # there are some exceptions to the rule because of the API
    if search in STRING_SEARCHES or (search == "title" and endpoint == "arcs"):
        search_value = "title" if search == "title" else search
        return f"{URL}{endpoint}/search/{search_value}/{resource}"

    if search in NO_SEARCH_ID_SEARCHES and endpoint not in ['boats', 'arcs']:
        return f"{URL}{endpoint}/{search[:-3]}/{resource}/"

    if search in ID_SEARCHES:
        return f"{URL}{endpoint}/search/{search[:-3]}/{resource}"

    if "title" in search:
        return (
            f"{URL}{endpoint}/search/{resource}"
            if search == "title" else
            f"{URL}{endpoint}/{search[:-6]}/search/{resource}"
        )

    if search == "destroy":
        return f"{URL}{endpoint}/{search}/{resource}"

    return f"{URL}{endpoint}/{resource}"


def convert_resource(resource):
    """
    Convert the resource to a valid url for exceptions

    This function is supposed to be used only by build_url

    :param resource: The resource to convert
    :return: The converted resource
    """
    if isinstance(resource, int):
        return resource
    # If the character name contains a slash, we only want the first part
    if ' / ' in resource:
        resource = resource.split(" / ")[0]
    return resource.replace(" ", "%20")


def pretty_print(data):
    """
    Pretty print the data
    """
    def convert_to_dict(obj):
        return obj.__dict__

    json_string = json.dumps(
        data,
        indent=4,
        ensure_ascii=False,
        default=convert_to_dict).encode("utf8").decode()
    return json_string



"""
One Piece API wrapper (french data)
"""
from .interface import API


def episode_by_id(resource):
    """
    Get episode by id

    See https://api-onepiece.com/documentation/5-episode for more information

    :param resource: id of the episode (int)
    :return: API unique object
    """
    return API("episodes", "id", resource)


def episode_by_title(resource):
    """
    Get episode by title

    See https://api-onepiece.com/documentation/5-episode for more information

    :param resource: title of the episode (str)
    :return: API list object (with episodes matching the title search)
    """
    return API("episodes", "title", resource)


def episode_by_saga_id(resource):
    """
    Get episode by saga id

    See https://api-onepiece.com/documentation/5-episode for more information

    :param resource: saga id of the episode (int)
    :return: API list object (with episodes matching the saga id search)
    """
    return API("episodes", "saga_id", resource)


def episode_by_arc_id(resource):
    """
    Get episode by arc id

    See https://api-onepiece.com/documentation/5-episode for more information

    :param resource: arc id of the episode (int)
    :return: API list object (with episodes matching the arc id search)
    """
    return API("episodes", "arc_id", resource)


def episode_by_arc_title(resource):
    """
    Get episode by arc title

    See https://api-onepiece.com/documentation/5-episode for more information

    :param resource: arc title of the episode (str)
    :return: API list object (with episodes matching the arc title search)
    """
    return API("episodes", "arc_title", resource)


def episode_by_saga_title(resource):
    """
    Get episode by saga title

    See https://api-onepiece.com/documentation/5-episode for more information

    :param resource: saga title of the episode (str)
    :return: API list object (with episodes matching the saga title search)
    """
    return API("episodes", "saga_title", resource)


def count_episodes():
    """
    Get count of episodes

    See https://api-onepiece.com/documentation/5-episode for more information

    :return: API unique object (with the count of episodes)
    """
    return API("episodes", "count")


def movie_by_id(resource):
    """
    Get movie by id

    See https://api-onepiece.com/documentation/7-movie for more information

    :param resource: id of the movie (int)
    :return: API unique object
    """
    return API("movies", "id", resource)


def movie_by_title(resource):
    """
    Get movie by title

    See https://api-onepiece.com/documentation/7-movie for more information

    :param resource: title of the movie (str)
    :return: API list object (with movies matching the title search)
    """
    return API("movies", "title", resource)


def count_movies():
    """
    Get count of movies

    See https://api-onepiece.com/documentation/7-movie for more information

    :return: API unique object (with the count of movies)
    """
    return API("movies", "count")


def tome_by_id(resource):
    """
    Get tome by id

    See https://api-onepiece.com/documentation/4-tome for more information

    :param resource: id of the tome (int)
    :return: API unique object
    """
    return API("tomes", "id", resource)


def tome_by_title(resource):
    """
    Get tome by title

    See https://api-onepiece.com/documentation/4-tome for more information

    :param resource: title of the tome (str)
    :return: API list object (with tomes matching the title search)
    """
    return API("tomes", "title", resource)


def count_tomes():
    """
    Get count of tomes

    See https://api-onepiece.com/documentation/4-tome for more information

    :return: API unique object (with the count of tomes)
    """
    return API("tomes", "count")


def chapter_by_id(resource):
    """
    Get chapter by id

    See https://api-onepiece.com/documentation/3-chapter for more information

    :param resource: id of the chapter (int)
    :return: API unique object
    """
    return API("chapters", "id", resource)


def chapter_by_title(resource):
    """
    Get chapter by title

    See https://api-onepiece.com/documentation/3-chapter for more information

    :param resource: title of the chapter (str)
    :return: API list object (with chapters matching the title search)
    """
    return API("chapters", "title", resource)


def chapter_by_tome_id(resource):
    """
    Get chapter by tome id

    See https://api-onepiece.com/documentation/3-chapter for more information

    :param resource: tome id of the chapter (int)
    :return: API list object (with chapters matching the tome id search)
    """
    return API("chapters", "tome_id", resource)


def chapter_by_tome(resource):
    """
    Get chapter by tome title

    See https://api-onepiece.com/documentation/3-chapter for more information

    :param resource: title of the tome (str)
    :return: API list object (with chapters matching the tome title search)
    """
    return API("chapters", "tome", resource)


def count_chapters():
    """
    Get count of chapters

    See https://api-onepiece.com/documentation/3-chapter for more information

    :return: API unique object (with the count of chapters)
    """
    return API("chapters", "count")


def arc_by_id(resource):
    """
    Get arc by id

    See https://api-onepiece.com/documentation/15-arc for more information

    :param resource: id of the arc (int)
    :return: API unique object
    """
    return API("arcs", "id", resource)


def arc_by_title(resource):
    """
    Get arc by title

    See https://api-onepiece.com/documentation/15-arc for more information

    :param resource: title of the arc (str)
    :return: API list object (with arcs matching the title search)
    """
    return API("arcs", "title", resource)


def arc_by_saga_id(resource):
    """
    Get arc by saga id

    See https://api-onepiece.com/documentation/15-arc for more information

    :param resource: saga id of the arc (int)
    :return: API list object (with arcs matching the saga id search)
    """
    return API("arcs", "saga_id", resource)


def count_arcs():
    """
    Get count of arcs

    See https://api-onepiece.com/documentation/15-arc for more information

    :return: API unique object (with the count of arcs)
    """
    return API("arcs", "count")


def saga_by_id(resource):
    """
    Get saga by id

    See https://api-onepiece.com/documentation/1-saga for more information

    :param resource: id of the saga (int)
    :return: API unique object
    """
    return API("sagas", "id", resource)


def saga_by_title(resource):
    """
    Get saga by title

    See https://api-onepiece.com/documentation/1-saga for more information

    :param resource: title of the saga (str)
    :return: API list object (with sagas matching the title search)
    """
    return API("sagas", "title", resource)


def count_sagas():
    """
    Get count of sagas

    See https://api-onepiece.com/documentation/1-saga for more information

    :return: API unique object (with the count of sagas)
    """
    return API("sagas", "count")


def haki_by_id(resource):
    """
    Get haki by id

    See https://api-onepiece.com/documentation/9-haki for more information

    :param resource: id of the haki (int)
    :return: API unique object
    """
    return API("hakis", "id", resource)


def haki_by_name(resource):
    """
    Get haki by name

    See https://api-onepiece.com/documentation/9-haki for more information

    :param resource: name of the haki (str)
    :return: API list object (with hakis matching the name search)
    """
    return API("hakis", "name", resource)


def haki_by_roman_name(resource):
    """
    Get haki by romanized name

    See https://api-onepiece.com/documentation/9-haki for more information

    :param resource: romanized name of the haki (str)
    :return: API list object (with hakis matching the romanized name search)
    """
    return API("hakis", "roman_name", resource)


def haki_by_characters_id(resource):
    """
    Get haki by character id

    See https://api-onepiece.com/documentation/9-haki for more information

    :param resource: id of the character who has the haki (int)
    :return: API list object (with hakis matching the character id search)
    """
    return API("hakis", "characters_id", resource)


def count_hakis():
    """
    Get count of hakis

    See https://api-onepiece.com/documentation/9-haki for more information

    :return: API unique object (with the count of hakis)
    """
    return API("hakis", "count")


def character_by_id(resource):
    """
    Get character by id

    See https://api-onepiece.com/documentation/13-character for more information

    :param resource: id of the character (int)
    :return: API unique object
    """
    return API("characters", "id", resource)


def character_by_name(resource):
    """
    Get character by name

    See https://api-onepiece.com/documentation/13-character for more information

    :param resource: name of the character (str)
    :return: API list object (with characters matching the name search)
    """
    return API("characters", "name", resource)


def character_by_job(resource):
    """
    Get character by job

    See https://api-onepiece.com/documentation/13-character for more information

    :param resource: job of the character (str)
    :return: API list object (with characters matching the job search)
    """
    return API("characters", "job", resource)


def character_by_bounty(resource):
    """
    Get character by bounty

    See https://api-onepiece.com/documentation/13-character for more information

    :param resource: bounty of the character (str) (ex: '1.000.000')
    :return: API list object (with characters matching the bounty search)
    """
    return API("characters", "bounty", resource)


def character_by_status(resource):
    """
    Get character by status

    See https://api-onepiece.com/documentation/13-character for more information

    :param resource: status of the character (str) (ex: 'vivant', 'mort')
    :return: API list object (with characters matching the status search)
    """
    return API("characters", "status", resource)


def character_by_size(resource):
    """
    Get character by size

    See https://api-onepiece.com/documentation/13-character for more information

    :param resource: size of the character (str) or (int) (ex: '180cm', 180)
    :return: API list object (with characters matching the size search)
    """
    return API("characters", "size", resource)


def character_by_crew_id(resource):
    """
    Get character by crew id

    See https://api-onepiece.com/documentation/13-character for more information

    :param resource: id of the crew of the character (int)
    :return: API list object (with characters matching the crew id search)
    """
    return API("characters", "crew_id", resource)


def character_by_fruit_id(resource):
    """
    Get character by fruit id

    See https://api-onepiece.com/documentation/13-character for more information

    :param resource: id of the fruit of the character (int)
    :return: API list object (with characters matching the fruit id search)
    """
    return API("characters", "fruit_id", resource)


def count_characters():
    """
    Get count of characters

    See https://api-onepiece.com/documentation/13-character for more information

    :return: API unique object (with the count of characters)
    """
    return API("characters", "count")


def dial_by_id(resource):
    """
    Get dial by id

    See https://api-onepiece.com/documentation/6-dial for more information

    :param resource: id of the dial (int)
    :return: API unique object
    """
    return API("dials", "id", resource)


def dial_by_name(resource):
    """
    Get dial by name

    See https://api-onepiece.com/documentation/6-dial for more information

    :param resource: name of the dial (str)
    :return: API list object (with dials matching the name search)
    """
    return API("dials", "name", resource)


def dial_by_type(resource):
    """
    Get dial by type

    See https://api-onepiece.com/documentation/6-dial for more information

    :param resource: type of the dial (str)
    :return: API list object (with dials matching the type search)
    """
    return API("dials", "type", resource)


def count_dials():
    """
    Get count of dials

    See https://api-onepiece.com/documentation/6-dial for more information

    :return: API unique object (with the count of dials)
    """
    return API("dials", "count")


def luffy_gear_by_id(resource):
    """
    Get luffy gear by id

    See https://api-onepiece.com/documentation/10-luffy-gear for more information

    :param resource: id of the luffy gear (int)
    :return: API unique object
    """
    return API("luffy/gears", "id", resource)


def luffy_gear_by_title(resource):
    """
    Get luffy gear by title

    See https://api-onepiece.com/documentation/10-luffy-gear for more information

    :param resource: title of the luffy gear (str)
    :return: API list object (with luffy gears matching the title search)
    """
    return API("luffy/gears", "title", resource)


def count_luffy_gears():
    """
    Get count of luffy gears

    See https://api-onepiece.com/documentation/10-luffy-gear for more information

    :return: API unique object (with the count of luffy gears)
    """
    return API("luffy/gears", "count")


def luffy_technique_by_id(resource):
    """
    Get luffy technique by id

    See https://api-onepiece.com/documentation/11-luffy-technique for more information

    :param resource: id of the luffy technique (int)
    :return: API unique object
    """
    return API("luffy/techniques", "id", resource)


def luffy_technique_by_name(resource):
    """
    Get luffy technique by name

    See https://api-onepiece.com/documentation/11-luffy-technique for more information

    :param resource: name of the luffy technique (str)
    :return: API list object (with luffy techniques matching the name search)
    """
    return API("luffy/techniques", "name", resource)


def luffy_technique_by_translation(resource):
    """
    Get luffy technique by translation

    See https://api-onepiece.com/documentation/11-luffy-technique for more information

    :param resource: translation of the luffy technique (str) (french translation)
    :return: API list object (with luffy techniques matching the translation search)
    """
    return API("luffy/techniques", "translation", resource)


def luffy_technique_by_gear_id(resource):
    """
    Get luffy technique by gear id

    See https://api-onepiece.com/documentation/11-luffy-technique for more information

    :param resource: id of the gear of the luffy technique (int)
    :return: API list object (with luffy techniques matching the gear id search)
    """
    return API("luffy/techniques", "gear_id", resource)


def count_luffy_techniques():
    """
    Get count of luffy techniques

    See https://api-onepiece.com/documentation/11-luffy-technique for more information

    :return: API unique object (with the count of luffy techniques)
    """
    return API("luffy/techniques", "count")


def locate_by_id(resource):
    """
    Get locate by id

    See https://api-onepiece.com/documentation/16-locate for more information

    :param resource: id of the locate (int)
    :return: API unique object
    """
    return API("locates", "id", resource)


def locate_by_name(resource):
    """
    Get locate by name

    See https://api-onepiece.com/documentation/16-locate for more information

    :param resource: name of the locate (str)
    :return: API list object (with locates matching the name search)
    """
    return API("locates", "name", resource)


def locate_by_sea(resource):
    """
    Get locate by sea

    See https://api-onepiece.com/documentation/16-locate for more information

    :param resource: sea of the locate (str)
    :return: API list object (with locates matching the sea search)
    """
    return API("locates", "sea", resource)


def locate_by_affiliation(resource):
    """
    Get locate by affiliation

    See https://api-onepiece.com/documentation/16-locate for more information

    :param resource: affiliation of the locate (str)
    :return: API list object (with locates matching the affiliation search)
    """
    return API("locates", "affiliation", resource)


def count_locates():
    """
    Get count of locates

    See https://api-onepiece.com/documentation/16-locate for more information

    :return: API unique object (with the count of locates)
    """
    return API("locates", "count")


def fruit_by_id(resource):
    """
    Get fruit by id

    See https://api-onepiece.com/documentation/2-fruit for more information

    :param resource: id of the fruit (int)
    :return: API unique object
    """
    return API("fruits", "id", resource)


def fruit_by_name(resource):
    """
    Get fruit by name

    See https://api-onepiece.com/documentation/2-fruit for more information

    :param resource: name of the fruit (str)
    :return: API list object (with fruits matching the name search)
    """
    return API("fruits", "name", resource)


def fruit_by_type(resource):
    """
    Get fruit by type

    See https://api-onepiece.com/documentation/2-fruit for more information

    :param resource: type of the fruit (str) (paramecia, zoan, zoan mythique, logia)
    :return: API list object (with fruits matching the type search)
    """
    return API("fruits", "type", resource)


def count_fruits():
    """
    Get count of fruits

    See https://api-onepiece.com/documentation/2-fruit for more information

    :return: API unique object (with the count of fruits)
    """
    return API("fruits", "count")


def sword_by_id(resource):
    """
    Get sword by id

    See https://api-onepiece.com/documentation/8-sword for more information

    :param resource: id of the sword (int)
    :return: API unique object
    """
    return API("swords", "id", resource)


def sword_by_name(resource):
    """
    Get sword by name

    See https://api-onepiece.com/documentation/8-sword for more information

    :param resource: name of the sword (str)
    :return: API list object (with swords matching the name search)
    """
    return API("swords", "name", resource)


def sword_by_destroy(resource):
    """
    Get sword by type

    See https://api-onepiece.com/documentation/8-sword for more information

    :param resource: type of the sword (int) (1 = is destroyed, 0 = is not destroyed)
    :return: API list object (with swords matching the type search)
    """
    return API("swords", "destroy", resource)


def count_swords():
    """
    Get count of swords

    See https: // api - onepiece.com / documentation / 8 - sword for more information

    : return: API unique object(with the count of swords)
    """
    return API("swords", "count")


def boat_by_id(resource):
    """
    Get boat by id

    See https: // api - onepiece.com / documentation / 14 - boat for more information

    : param resource: id of the boat(int)
    : return: API unique object
    """
    return API("boats", "id", resource)


def boat_by_name(resource):
    """
    Get boat by name

    See https: // api - onepiece.com / documentation / 14 - boat for more information

    : param resource: name of the boat(str)
    : return: API list object(with boats matching the name search)
    """
    return API("boats", "name", resource)


def boat_by_type(resource):
    """
    Get boat by type

    See https: // api - onepiece.com / documentation / 14 - boat for more information

    : param resource: type of the boat(str)(pirate, gouvernement)
    : return: API list object(with boats matching the type search)
    """
    return API("boats", "type", resource)


def boat_by_crew_id(resource):
    """
    Get boat by crew id

    See https: // api - onepiece.com / documentation / 14 - boat for more information

    : param resource: crew id of the boat(int)
    : return: API list object(with boats matching the crew id search)
    """
    return API("boats", "crew_id", resource)


def boat_by_captain_id(resource):
    """
    Get boat by captain id

    See https: // api - onepiece.com / documentation / 14 - boat for more information

    : param resource: captain id of the boat(int)
    : return: API list object(with boats matching the captain id search)
    """
    return API("boats", "captain_id", resource)


def count_boats():
    """
    Get count of boats

    See https: // api - onepiece.com / documentation / 14 - boat for more information

    : return: API unique object(with the count of boats)
    """
    return API("boats", "count")


def crew_by_id(resource):
    """
    Get crew by id

    See https: // api - onepiece.com / documentation / 12 - crew for more information

    : param resource: id of the crew(int)
    : return: API unique object
    """
    return API("crews", "id", resource)


def crew_by_name(resource):
    """
    Get crew by name

    See https: // api - onepiece.com / documentation / 12 - crew for more information

    : param resource: name of the crew(str)
    : return: API list object(with crews matching the name search)
    """
    return API("crews", "name", resource)


def crew_by_status(resource):
    """
    Get crew by status

    See https: // api - onepiece.com / documentation / 12 - crew for more information

    : param resource: status of the crew(str)(inconnu, actif, inactif)
    : return: API list object(with crews matching the status search)
    """
    return API("crews", "status", resource)


def yonko_crews():
    """
    Get yonko crews

    See https: // api - onepiece.com / documentation / 12 - crew for more information

    : return: API list object(with crews which are yonko)
    """
    return API("crews", "yonko")


def count_crews():
    """
    Get count of crews

    See https: // api - onepiece.com / documentation / 12 - crew for more information

    : return: API unique object(with the count of crews)
    """
    return API("crews", "count")



import setuptools

with open("README.md", "r", encoding="utf-8") as file:
    long_description = file.read()

setuptools.setup(
    name="onepyece",
    version="1.3.1",
    author="Rémi JARA",
    author_email="remi.jara4@gmail.com",
    description="A package to use the One-Piece API : https://api-onepiece.com/",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/icepick4/onepyece",
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=["onepyece"],
    python_requires=">=3.8",
    install_requires=["requests"],
    license="MIT",
    licence_files=["LICENSE"],
    keywords="one-piece api wrapper onepyece onepiece onepiece-api onepiece-api-wrapper manga anime anime-manga",
)



"""
Mother class of the API wrapper
"""
from .api import get_data
from .common import build_url, pretty_print


class API:
    """
    The API class

    This class is used to get data from the API and exploit data from it

    :param endpoint: The endpoint to get the data from
    :param search_term: The search to get the data from
    :param resource: The resource to get the data from
    :param object: If the object is an object or not
    :param data: The data object to use to create the object

    See https://api-onepiece.com/documentation for more information about URL
    """

    def __init__(self, endpoint=None, search_term=None, resource=None, data=None):
        self.endpoint = endpoint
        self.search_term = search_term
        self.resource = resource
        if self.endpoint is not None:
            self.url = build_url(endpoint, search_term, resource)
        if data is None:
            self.__load()
        else:
            self.url = None
            self.update_dict_recursively(data)

    def __repr__(self):
        if self.url is None:
            return f"API(object={self.__dict__})"
        if self.resource is not None:
            return (
                f"API(endpoint='{self.endpoint}',"
                f"search='{self.search_term}',"
                f"resource='{self.resource}',"
                f"url='{self.url}')"
            )
        if self.search_term is not None:
            return f"API(endpoint='{self.endpoint}', search='{self.search_term}', url='{self.url}')"
        return f"API(endpoint='{self.endpoint}', url='{self.url}')"

    def __str__(self):
        if getattr(self, "count", None) is not None:
            return f"Total {self.endpoint} found for this search: {self.count}."
        return pretty_print(self.__dict__)

    def __iter__(self):
        if getattr(self, "results", None) is not None:
            return iter(self.results)
        raise TypeError(f"Object of type {type(self)} is not iterable")

    def __getitem__(self, item):
        if getattr(self, "results", None) is not None:
            return self.results[item]
        raise TypeError(f"Object of type {type(self)} has no __getitem__ for unique result")

    def __len__(self):
        if getattr(self, "count", None) is not None:
            return self.count
        raise TypeError(f"Object of type {type(self)} has no len() for unique result")

    def __load(self):
        """
        Load the data from the API

        Set the attributes of the object with the data from the API

        :return: None
        """
        api_data = get_data(self.url)
        if api_data is None:
            raise ValueError(
                f"No data found for the given search:{self.search_term}"
                f" and resource_id:{self.resource}"
            )
        if isinstance(api_data, list) and len(api_data) > 1:
            self.results = [API(data=result) for result in api_data]
            self.count = len(api_data)
        elif isinstance(api_data, list) and len(api_data) == 1:
            self.__dict__.update(api_data[0])
        elif isinstance(api_data, dict):
            self.update_dict_recursively(api_data)
        elif isinstance(api_data, int):
            self.count = api_data
        else:
            raise ValueError(f"Unknown data type: {type(api_data)}")

    def update_dict_recursively(self, data):
        """
        Update the dict of the object recursively

        :param data: The data to update the dict with
        :return: None
        """
        for key, value in data.items():
            if isinstance(value, dict):
                self.__dict__[key] = API(data=value)
            elif isinstance(value, list):
                self.__dict__[key] = API(data=value[0])
            else:
                self.__dict__[key] = value

    def search(self, search, resource=None):
        """
        Search for a specific search term after initializing the API object
        """
        if self.search_term is None and self.resource is None:
            return API(self.endpoint, search, resource)
        raise ValueError(
            "You can't instantiate a search API object because you already have"
            f"{self.search_term} as search and {self.resource} as resource"
        )

    def edit_resource(self, resource):
        """
        Edit the resource of the API object
        Use this method after using the search method
        """
        if self.search_term is None:
            raise ValueError('You can\'t change the resource without'
                             ' search use the method search() first'
                             )
        temp_api = API(self.endpoint, self.search_term, resource)
        self.__dict__.update(temp_api.__dict__)
